import {select, 
        csv, 
        scaleLinear,
        scaleTime,
        max,
      	axisLeft,
       	axisBottom,
       	extent,
        area,
        curveBasis
} from 'd3';

const svg = select('svg');

const height =+svg.attr('height');
const width =+svg.attr('width');

/*

*/

const render = data => {
  //SPECIFIC TO DATA SET, CHANGE XVAL AND YVAL 
  const xValue = d => d.timestamp;
  const xAxisLabel = 'time';
  
  const yValue = d => d.temperature;
  const circleRadius = 3; 
  const yAxisLabel = 'temperature';
  
  const title = 'Temperature in San Francisco in a Week';
  

  const margin = { top: 50, right: 60, bottom: 70, left: 95 };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

  //START - GENERALIZED CODE TO DATA SET
  const xScale = scaleTime() //create instance of scaleLinear
  	.domain(extent(data, xValue)) //returns array of range from [min, max]
  	.range([0, innerWidth]);  
  	//no nice here cus it cuts off the ends of x domain
  
  const yScale = scaleLinear()
  	.domain(extent(data, yValue))
  	.range([innerHeight, 0])
  	.nice();
  
  
  const yAxis = axisLeft(yScale)
  							.tickSize(-innerWidth)
  							.tickPadding(10);
  
  const xAxis = axisBottom(xScale)
  							.ticks(6) //specify num of ticks on xAxis
  							.tickSize(-innerHeight) //makes ticks extend all the way up
  							.tickPadding(10);
  
  const group = svg.append('g')
  	.attr('transform', `translate(${margin.left},${margin.top})`);
  
  const yAxisGroup = group.append('g')
    .call(yAxis)
  
  yAxisGroup.selectAll('.domain') //selector strings like css and html class selectors
    	.remove(); 
  
  yAxisGroup.append('text')
  	.attr('class', 'axis-label')
  	.attr('x', -innerHeight/2)
  	.attr('y', -35)
  	.attr('transform', 'rotate(-90)')
  	.attr('text-anchor', 'middle') // svg text align
    .attr('fill', 'black')
  	.text(yAxisLabel)
  
  const xAxisGroup = group.append('g')
    .call(xAxis)
  	.attr('transform', `translate(0,${innerHeight})`);
  
  xAxisGroup.select('.domain').remove();
  
  xAxisGroup.append('text')
  	.attr('class', 'axis-label')
  	.attr('x', innerWidth/2)
  	.attr('y', 40)
    .attr('fill', 'black')
  	.text(xAxisLabel)
  
  const areaMaker = area() //values require pixel coordinates to see where to draw
  	.x(d => xScale(xValue(d)))
  	.y0(innerHeight) //check docs on area to see y0
  	.y1(d => yScale(yValue(d))) //docs for y1
  	.curve(curveBasis); //cubic basis spline
  
  group.append('path')
  	.attr('class', 'line-path')
  	.attr('d', areaMaker(data));
  
  //comment out if dont want circles
// 	group.selectAll('circle').data(data) //data join elements of data to DOM rect
//   	.enter().append('circle') 
//   		.attr('cy', d => yScale(yValue(d)))
//   		.attr('cx', d => xScale(xValue(d))) //pass in 1 row of data, specifically population
//   		.attr('r', circleRadius); // bandwidth = computed width of single bar
  
  group.append('text')
  	.attr('class', 'graph-title')
  	.attr('y', -10)
  	.text(title);
  //END - GENERALIZED CODE
};


/*
returns a promise to be loaded -> then
xml http request, load csv string, parses csv string -> array of objs
key of each obj = col, val = val
*/
csv('https://vizhub.com/curran/datasets/temperature-in-san-francisco.csv')
  .then(data => {
    data.forEach(d =>{
      //unary operator parses string -> values, data in thousands so *1000
      d.temperature = +d.temperature; 
      d.timestamp = new Date(d.timestamp); 
    });
    // console.log(data);
    render(data);
  });  